@@ -66,10 +66,9 @@ static struct ipt_log_names ipt_log_names[]
 static u_int8_t
 parse_level(const char *level)
 {
-	int lev;
+	unsigned int lev;
 
-	lev = string_to_number(level, 0, 7);
-	if (lev == -1) {
+	if (string_to_number(level, 0, 7, &lev) == -1) {
 		unsigned int i = 0;
 
 		for (i = 0;
@@ -240,6 +239,7 @@ save(const struct ipt_ip *ip, const struct ipt_entry_target *target)
 		printf("--log-ip-options ");
 }
 
+static
 struct iptables_target log
 = { NULL,
     "LOG",
