@@ -44,7 +44,7 @@ init(struct ipt_entry_match *m, unsigned int *nfcache)
 static void
 parse_string(const unsigned char *s, struct ipt_string_info *info)
 {	
-        if (strlen(s) <= BM_MAX_LEN) strcpy(info->string, s);
+        if (strlen(s) <= BM_MAX_NLEN) strcpy(info->string, s);
 	else exit_error(PARAMETER_PROBLEM, "STRING too long `%s'", s);
 }
 
@@ -113,6 +113,7 @@ save(const struct ipt_ip *ip, const struct ipt_entry_match *match)
 		  ((struct ipt_string_info *)match->data)->invert, 0);
 }
 
+static
 struct iptables_match string
 = { NULL,
     "string",
