@@ -51,6 +51,9 @@ struct iptables_match
 	struct ipt_entry_match *m;
 	unsigned int mflags;
 	unsigned int used;
+#ifdef NO_SHARED_LIBS
+	unsigned int loaded; /* simulate loading so options are merged properly */
+#endif
 };
 
 struct iptables_target
@@ -98,6 +101,9 @@ struct iptables_target
 	struct ipt_entry_target *t;
 	unsigned int tflags;
 	unsigned int used;
+#ifdef NO_SHARED_LIBS
+	unsigned int loaded; /* simulate loading so options are merged properly */
+#endif
 };
 
 /* Your shared library should call one of these. */
