@@ -51,7 +51,7 @@ print_iface(char letter, const char *iface, const unsigned char *mask,
 			if (iface[i] != '\0')
 				printf("%c", iface[i]);
 		} else {
-			if (iface[i] != '\0')
+			if (iface[i] == '\0')
 				printf("+");
 			break;
 		}
@@ -69,7 +69,7 @@ struct pprot {
 static const struct pprot chain_protos[] = {
 	{ "tcp", IPPROTO_TCP },
 	{ "udp", IPPROTO_UDP },
-	{ "icmp", IPPROTO_ICMP },
+	{ "icmpv6", IPPROTO_ICMPV6 },
 	{ "esp", IPPROTO_ESP },
 	{ "ah", IPPROTO_AH },
 };
@@ -155,6 +155,7 @@ static void print_rule(const struct ip6t_entry *e,
 		ip6tc_handle_t *h, int counters)
 {
 	struct ip6t_entry_target *t;
+	const char *target_name;
 
 	/* print counters */
 	if (counters)
@@ -196,7 +197,9 @@ static void print_rule(const struct ip6t_entry *e,
 	}
 
 	/* Print target name */	
-	printf("-j %s ", ip6tc_get_target(e, h));
+	target_name = ip6tc_get_target(e, h);
+	if (target_name && *target_name != '\0')
+		printf("-j %s ", ip6tc_get_target(e, h));
 
 	/* Print targinfo part */
 	t = ip6t_get_target((struct ip6t_entry *)e);
@@ -311,6 +314,10 @@ int main(int argc, char *argv[])
 	program_name = "ip6tables-save";
 	program_version = NETFILTER_VERSION;
 
+#ifdef NO_SHARED_LIBS
+	init_extensions();
+#endif
+
 	while ((c = getopt_long(argc, argv, "bc", options, NULL)) != -1) {
 		switch (c) {
 		case 'b':
