@@ -135,7 +135,7 @@ static struct option original_opts[] = {
 
 #ifndef __OPTIMIZE__
 struct ip6t_entry_target *
-ip6t_get_target(struct ipt_entry *e)
+ip6t_get_target(struct ip6t_entry *e)
 {
 	        return (void *)e + e->target_offset;
 }
@@ -332,7 +332,7 @@ exit_printhelp(void)
 "  --table	-t table	table to manipulate (default: `filter')\n"
 "  --verbose	-v		verbose mode\n"
 "  --exact	-x		expand numbers (display exact values)\n"
-"[!] --fragment	-f		match second or further fragments only\n"
+/*"[!] --fragment	-f		match second or further fragments only\n"*/
 "[!] --version	-V		print package version.\n");
 
 	/* Print out any special helps. A user might like to be able to add a --help 
@@ -452,71 +452,103 @@ fw_malloc(size_t size)
 	return p;
 }
 
-static struct in6_addr *
-host_to_addr(const char *name, unsigned int *naddr)
-{
-#if 0
-	/* TODO: getipnodebyname() missing */
-	struct hostent *host;
-	struct in6_addr *addr;
-	unsigned int i;
-
-	*naddr = 0;
-	if ((host = getipnodebyname(name, AF_INET6, 0, NULL)) != NULL) {
-		if (host->h_length != sizeof(struct in6_addr))
-			return (struct in6_addr *) NULL;
-
-		while (host->h_addr_list[*naddr] != (char *) NULL)
-			(*naddr)++;
-		addr = fw_calloc(*naddr, sizeof(struct in6_addr));
-		for (i = 0; i < *naddr; i++)
-			in6addrcpy(&(addr[i]),
-				  (struct in6_addr *) host->h_addr_list[i]);
-		freehostent(host);
-		return addr;
-	}
-#endif
-
-	return (struct in6_addr *) NULL;
-}
-
-static char *
-addr_to_host(const struct in6_addr *addr)
-{
-#if 0
-	/* TODO: getipnodebyaddr missing */
-	struct hostent *host;
-
-	if ((host = getipnodebyaddr((char *) addr,
-				  sizeof(struct in6_addr), AF_INET6, NULL)) != NULL)
-		return (char *) host->h_name;
-	/*TODO: freehostent()*/
-#endif
-
-	return (char *) NULL;
-}
-
 static char *
 addr_to_numeric(const struct in6_addr *addrp)
 {
-	// static char buf[20];
-	// ADDR tags + sep. + \0
-	// 0000:0000:0000:0000:0000:0000:0000:000.000.000.000
-	// (but inet_ntop() supports only 
-	// 0000:0000:0000:0000:0000:0000:0000:0000 format now.)
+	/* 0000:0000:0000:0000:0000:000.000.000.000
+	 * 0000:0000:0000:0000:0000:0000:0000:0000 */
 	static char buf[50+1];
-	return (char *)inet_ntop(AF_INET6, addrp, buf, sizeof buf);
+	return (char *)inet_ntop(AF_INET6, addrp, buf, sizeof(buf));
 }
 
 static struct in6_addr *
 numeric_to_addr(const char *num)
 {
 	static struct in6_addr ap;
-	if (inet_pton(AF_INET6, num, &ap) == 1)
+	int err;
+	if ((err=inet_pton(AF_INET6, num, &ap)) == 1)
 		return &ap;
+#ifdef DEBUG
+	fprintf(stderr, "\nnumeric2addr: %d\n", err);
+#endif
 	return (struct in6_addr *)NULL;
 }
 
+
+static struct in6_addr *
+host_to_addr(const char *name, unsigned int *naddr)
+{
+	struct addrinfo hints;
+        struct addrinfo *res;
+        static struct in6_addr *addr;
+	int err;
+
+	memset(&hints, 0, sizeof(hints));
+        hints.ai_flags=AI_CANONNAME;
+        hints.ai_family=AF_INET6;
+        hints.ai_socktype=SOCK_RAW;
+        hints.ai_protocol=41;
+        hints.ai_next=NULL;
+
+	*naddr = 0;
+        if ( (err=getaddrinfo(name, NULL, &hints, &res)) != 0 ){
+#ifdef DEBUG
+                fprintf(stderr,"Name2IP: %s\n",gai_strerror(err)); 
+#endif
+                return (struct in6_addr *) NULL;
+        } else {
+		if (res->ai_family != AF_INET6 ||
+		    res->ai_addrlen != sizeof(struct sockaddr_in6))
+			return (struct in6_addr *) NULL;
+
+#ifdef DEBUG
+                fprintf(stderr, "resolved: len=%d  %s ", res->ai_addrlen, 
+                    addr_to_numeric(&(((struct sockaddr_in6 *)res->ai_addr)->sin6_addr)));
+#endif
+		/* Get the first element of the address-chain */
+		addr = fw_calloc(1, sizeof(struct in6_addr));
+		in6addrcpy(addr, (struct in6_addr *)
+			&((struct sockaddr_in6 *)res->ai_addr)->sin6_addr);
+		freeaddrinfo(res);
+		*naddr = 1;
+		return addr;
+	}
+
+	return (struct in6_addr *) NULL;
+}
+
+/* XXX: the getnameinfo() returns "ai_family not supported".
+ *      it can be a glibc error. */
+static char *
+addr_to_host(const struct in6_addr *addr)
+{
+	struct sockaddr_in6 saddr;
+	int err;
+	static char hostname[NI_MAXHOST];
+
+	memset(&saddr, 0, sizeof(struct sockaddr_in6));
+	in6addrcpy(&(saddr.sin6_addr),(struct in6_addr *)addr);
+	saddr.sin6_family = AF_INET6;
+
+        if ( (err=getnameinfo((struct sockaddr *)&saddr,
+			       sizeof(struct sockaddr_in6),
+			       hostname, sizeof(hostname)-1,
+			       NULL, 0, 0)) != 0 ){
+#ifdef DEBUG
+                fprintf(stderr,"IP2Name: %s\n",gai_strerror(err)); 
+#endif
+                return (char *) NULL;
+        } else {
+#ifdef DEBUG
+		fprintf (stderr, "\naddr2host: %s\n", hostname);
+#endif
+
+		return hostname;
+	}
+
+	return (char *) NULL;
+}
+
 static char *
 mask_to_numeric(const struct in6_addr *addrp)
 {
@@ -531,7 +563,10 @@ mask_to_numeric(const struct in6_addr *addrp)
 static struct in6_addr *
 network_to_addr(const char *name)
 {
-	abort();
+	/*	abort();*/
+	/* TODO: not implemented yet, but the exception breaks the
+	 *       name resolvation */
+	return (struct in6_addr *)NULL;
 }
 
 static char *
@@ -577,7 +612,7 @@ parse_mask(char *mask)
 {
 	static struct in6_addr maskaddr;
 	struct in6_addr *addrp;
-	int bits;
+	unsigned int bits;
 
 	if (mask == NULL) {
 		/* no mask at all defaults to 128 bits */
@@ -586,7 +621,7 @@ parse_mask(char *mask)
 	}
 	if ((addrp = numeric_to_addr(mask)) != NULL)
 		return addrp;
-	if ((bits = string_to_number(mask, 0, 128)) == -1)
+	if (string_to_number(mask, 0, 128, &bits) == -1)
 		exit_error(PARAMETER_PROBLEM,
 			   "invalid mask `%s' specified", mask);
 	if (bits != 0) {
@@ -630,7 +665,7 @@ parse_hostnetworkmask(const char *name, struct in6_addr **addrpp,
 			addrp[j].in6_u.u6_addr32[k] &= maskp->in6_u.u6_addr32[k];
 		j++;
 		for (k = 0; k < j - 1; k++) {
-			if (!memcmp(&addrp[k], &addrp[j - 1], sizeof(struct in6_addr))) {
+			if (IN6_ARE_ADDR_EQUAL(&addrp[k], &addrp[j - 1])) {
 				(*naddrs)--;
 				j--;
 				break;
@@ -649,6 +684,7 @@ find_match(const char *name, enum ip6t_tryload tryload)
 			break;
 	}
 
+#ifndef NO_SHARED_LIBS
 	if (!ptr && tryload != DONT_LOAD) {
 		char path[sizeof(IP6T_LIB_DIR) + sizeof("/libip6t_.so")
 			 + strlen(name)];
@@ -666,6 +702,14 @@ find_match(const char *name, enum ip6t_tryload tryload)
 			exit_error(PARAMETER_PROBLEM,
 				   "Couldn't load match `%s'\n", name);
 	}
+#else
+	if (ptr && !ptr->loaded) {
+		if (tryload != DONT_LOAD)
+			ptr->loaded = 1;
+		else
+			ptr = NULL;
+	}
+#endif
 
         if (ptr)
                 ptr->used = 1;
@@ -678,10 +722,9 @@ find_match(const char *name, enum ip6t_tryload tryload)
 static struct ip6tables_match *
 find_proto(const char *pname, enum ip6t_tryload tryload, int nolookup)
 {
-	int proto;
+	unsigned int proto;
 
-	proto = string_to_number(pname, 0, 255);
-	if (proto != -1) 
+	if (string_to_number(pname, 0, 255, &proto) != -1) 
 		return find_match(proto_to_name(proto, nolookup), tryload);
 
 	return find_match(pname, tryload);
@@ -690,9 +733,9 @@ find_proto(const char *pname, enum ip6t_tryload tryload, int nolookup)
 static u_int16_t
 parse_protocol(const char *s)
 {
-	int proto = string_to_number(s, 0, 255);
+	unsigned int proto;
 
-	if (proto == -1) {
+	if (string_to_number(s, 0, 255, &proto) == -1) {
 		struct protoent *pent;
 
 		if ((pent = getprotobyname(s)))
@@ -745,7 +788,7 @@ parse_interface(const char *arg, char *vianame, unsigned char *mask)
 		memset(mask + vialen + 1, 0, IFNAMSIZ - vialen - 1);
 	}
 	for (i = 0; vianame[i]; i++) {
-		if (!isalnum(vianame[i])) {
+		if (!isalnum(vianame[i]) && vianame[i] != '_') {
 			printf("Warning: wierd character in interface"
 			       " `%s' (No aliases, :, ! or *).\n",
 			       vianame);
@@ -758,9 +801,9 @@ parse_interface(const char *arg, char *vianame, unsigned char *mask)
 static int
 parse_rulenumber(const char *rule)
 {
-	int rulenum = string_to_number(rule, 1, INT_MAX);
+	unsigned int rulenum; 
 
-	if (rulenum == -1)
+	if (string_to_number(rule, 1, INT_MAX, &rulenum))
 		exit_error(PARAMETER_PROBLEM,
 			   "Invalid rule number `%s'", rule);
 
@@ -789,18 +832,21 @@ parse_target(const char *targetname)
 }
 
 int
-string_to_number(const char *s, int min, int max)
+string_to_number(const char *s, unsigned int min, unsigned int max,
+		 unsigned int *ret)
 {
-       long number;
+	long number;
 	char *end;
 
 	/* Handle hex, octal, etc. */
-       errno = 0;
-       number = strtol(s, &end, 0);
+	errno = 0;
+	number = strtol(s, &end, 0);
 	if (*end == '\0' && end != s) {
 		/* we parsed a number, let's see if we want this */
-               if (errno != ERANGE && min <= number && number <= max)
-			return number;
+               if (errno != ERANGE && min <= number && number <= max) {
+			*ret = number;
+			return 0;
+	       }
 	}
 	return -1;
 }
@@ -844,6 +890,7 @@ find_target(const char *name, enum ip6t_tryload tryload)
 			break;
 	}
 
+#ifndef NO_SHARED_LIBS
 	if (!ptr && tryload != DONT_LOAD) {
 		char path[sizeof(IP6T_LIB_DIR) + sizeof("/libip6t_.so")
 			 + strlen(name)];
@@ -861,6 +908,14 @@ find_target(const char *name, enum ip6t_tryload tryload)
 				   "Couldn't load target `%s'%s\n", 
 				   name, dlerror());
 	}
+#else
+	if (ptr && !ptr->loaded) {
+		if (tryload != DONT_LOAD)
+			ptr->loaded = 1;
+		else
+			ptr = NULL;
+	}
+#endif
 
         if (ptr)
                 ptr->used = 1;
@@ -1511,7 +1566,7 @@ static char *get_modprobe(void)
         return NULL;
 }
 
-static int ip6tables_insmod(const char *modname, const char *modprobe)
+int ip6tables_insmod(const char *modname, const char *modprobe)
 {
         char *buf = NULL;
         char *argv[3];
