@@ -1,5 +1,5 @@
 /* Code to save the iptables state, in human readable-form. */
-/* Authors: Paul 'Rusty' Russel <rusty@linuxcare.com.au> and
+/* Authors: Paul 'Rusty' Russell <rusty@rustcorp.com.au> and
  * 	    Harald Welte <laforge@gnumonks.org>
  */
 #include <getopt.h>
@@ -148,6 +148,7 @@ static void print_rule(const struct ipt_entry *e,
 		iptc_handle_t *h, const char *chain, int counters)
 {
 	struct ipt_entry_target *t;
+	const char *target_name;
 
 	/* print counters */
 	if (counters)
@@ -181,7 +182,9 @@ static void print_rule(const struct ipt_entry *e,
 	}
 
 	/* Print target name */	
-	printf("-j %s ", iptc_get_target(e, h));
+	target_name = iptc_get_target(e, h);
+	if (target_name && (*target_name != '\0'))
+		printf("-j %s ", target_name);
 
 	/* Print targinfo part */
 	t = ipt_get_target((struct ipt_entry *)e);
@@ -303,6 +306,10 @@ int main(int argc, char *argv[])
 	program_name = "iptables-save";
 	program_version = NETFILTER_VERSION;
 
+#ifdef NO_SHARED_LIBS
+	init_extensions();
+#endif
+
 	while ((c = getopt_long(argc, argv, "bc", options, NULL)) != -1) {
 		switch (c) {
 		case 'b':
