@@ -250,7 +250,8 @@ dotted_to_addr(const char *dotted)
 	static struct in_addr addr;
 	unsigned char *addrp;
 	char *p, *q;
-	int onebyte, i;
+	unsigned int onebyte;
+	int i;
 	char buf[20];
 
 	/* copy dotted string, because we need to modify it */
@@ -263,7 +264,7 @@ dotted_to_addr(const char *dotted)
 			return (struct in_addr *) NULL;
 
 		*q = '\0';
-		if ((onebyte = string_to_number(p, 0, 255)) == -1)
+		if (string_to_number(p, 0, 255, &onebyte) == -1)
 			return (struct in_addr *) NULL;
 
 		addrp[i] = (unsigned char) onebyte;
@@ -271,7 +272,7 @@ dotted_to_addr(const char *dotted)
 	}
 
 	/* we've checked 3 bytes, now we check the last one */
-	if ((onebyte = string_to_number(p, 0, 255)) == -1)
+	if (string_to_number(p, 0, 255, &onebyte) == -1)
 		return (struct in_addr *) NULL;
 
 	addrp[3] = (unsigned char) onebyte;
@@ -581,7 +582,7 @@ parse_mask(char *mask)
 {
 	static struct in_addr maskaddr;
 	struct in_addr *addrp;
-	int bits;
+	unsigned int bits;
 
 	if (mask == NULL) {
 		/* no mask at all defaults to 32 bits */
@@ -591,7 +592,7 @@ parse_mask(char *mask)
 	if ((addrp = dotted_to_addr(mask)) != NULL)
 		/* dotted_to_addr already returns a network byte order addr */
 		return addrp;
-	if ((bits = string_to_number(mask, 0, 32)) == -1)
+	if (string_to_number(mask, 0, 32, &bits) == -1)
 		exit_error(PARAMETER_PROBLEM,
 			   "invalid mask `%s' specified", mask);
 	if (bits != 0) {
@@ -648,6 +649,7 @@ find_match(const char *name, enum ipt_tryload tryload)
 			break;
 	}
 
+#ifndef NO_SHARED_LIBS
 	if (!ptr && tryload != DONT_LOAD) {
 		char path[sizeof(IPT_LIB_DIR) + sizeof("/libipt_.so")
 			 + strlen(name)];
@@ -666,6 +668,14 @@ find_match(const char *name, enum ipt_tryload tryload)
 				   "Couldn't load match `%s':%s\n",
 				   name, dlerror());
 	}
+#else
+	if (ptr && !ptr->loaded) {
+		if (tryload != DONT_LOAD)
+			ptr->loaded = 1;
+		else
+			ptr = NULL;
+	}
+#endif
 
 	if (ptr)
 		ptr->used = 1;
@@ -677,10 +687,9 @@ find_match(const char *name, enum ipt_tryload tryload)
 static struct iptables_match *
 find_proto(const char *pname, enum ipt_tryload tryload, int nolookup)
 {
-	int proto;
+	unsigned int proto;
 
-	proto = string_to_number(pname, 0, 255);
-	if (proto != -1)
+	if (string_to_number(pname, 0, 255, &proto) != -1)
 		return find_match(proto_to_name(proto, nolookup), tryload);
 
 	return find_match(pname, tryload);
@@ -689,9 +698,9 @@ find_proto(const char *pname, enum ipt_tryload tryload, int nolookup)
 static u_int16_t
 parse_protocol(const char *s)
 {
-	int proto = string_to_number(s, 0, 255);
+	unsigned int proto;
 
-	if (proto == -1) {
+	if (string_to_number(s, 0, 255, &proto) == -1) {
 		struct protoent *pent;
 
 		if ((pent = getprotobyname(s)))
@@ -736,19 +745,18 @@ parse_interface(const char *arg, char *vianame, unsigned char *mask)
 	else if (vianame[vialen - 1] == '+') {
 		memset(mask, 0xFF, vialen - 1);
 		memset(mask + vialen - 1, 0, IFNAMSIZ - vialen + 1);
-		/* Remove `+' */
-		vianame[vialen - 1] = '\0';
+		/* Don't remove `+' here! -HW */
 	} else {
 		/* Include nul-terminator in match */
 		memset(mask, 0xFF, vialen + 1);
 		memset(mask + vialen + 1, 0, IFNAMSIZ - vialen - 1);
-	}
-	for (i = 0; vianame[i]; i++) {
-		if (!isalnum(vianame[i])) {
-			printf("Warning: wierd character in interface"
-			       " `%s' (No aliases, :, ! or *).\n",
-			       vianame);
-			break;
+		for (i = 0; vianame[i]; i++) {
+			if (!isalnum(vianame[i]) && vianame[i] != '_') {
+				printf("Warning: wierd character in interface"
+				       " `%s' (No aliases, :, ! or *).\n",
+				       vianame);
+				break;
+			}
 		}
 	}
 }
@@ -757,9 +765,9 @@ parse_interface(const char *arg, char *vianame, unsigned char *mask)
 static int
 parse_rulenumber(const char *rule)
 {
-	int rulenum = string_to_number(rule, 1, INT_MAX);
+	unsigned int rulenum;
 
-	if (rulenum == -1)
+	if (string_to_number(rule, 1, INT_MAX, &rulenum) == -1)
 		exit_error(PARAMETER_PROBLEM,
 			   "Invalid rule number `%s'", rule);
 
@@ -847,7 +855,8 @@ mask_to_dotted(const struct in_addr *mask)
 }
 
 int
-string_to_number(const char *s, int min, int max)
+string_to_number(const char *s, unsigned int min, unsigned int max,
+		 unsigned int *ret)
 {
 	long number;
 	char *end;
@@ -857,8 +866,10 @@ string_to_number(const char *s, int min, int max)
 	number = strtol(s, &end, 0);
 	if (*end == '\0' && end != s) {
 		/* we parsed a number, let's see if we want this */
-		if (errno != ERANGE && min <= number && number <= max)
-			return number;
+		if (errno != ERANGE && min <= number && number <= max) {
+			*ret = number;
+			return 0;
+		}
 	}
 	return -1;
 }
@@ -902,6 +913,7 @@ find_target(const char *name, enum ipt_tryload tryload)
 			break;
 	}
 
+#ifndef NO_SHARED_LIBS
 	if (!ptr && tryload != DONT_LOAD) {
 		char path[sizeof(IPT_LIB_DIR) + sizeof("/libipt_.so")
 			 + strlen(name)];
@@ -919,6 +931,14 @@ find_target(const char *name, enum ipt_tryload tryload)
 				   "Couldn't load target `%s':%s\n",
 				   name, dlerror());
 	}
+#else
+	if (ptr && !ptr->loaded) {
+		if (tryload != DONT_LOAD)
+			ptr->loaded = 1;
+		else
+			ptr = NULL;
+	}
+#endif
 
 	if (ptr)
 		ptr->used = 1;
@@ -1020,7 +1040,11 @@ print_num(u_int64_t number, unsigned int format)
 				number = (number + 500) / 1000;
 				if (number > 9999) {
 					number = (number + 500) / 1000;
-					printf(FMT("%4lluG ","%lluG "),number);
+					if (number > 9999) {
+						number = (number + 500) / 1000;
+						printf(FMT("%4lluT ","%lluT "), number);
+					}
+					else printf(FMT("%4lluG ","%lluG "), number);
 				}
 				else printf(FMT("%4lluM ","%lluM "), number);
 			} else
@@ -1165,10 +1189,6 @@ print_firewall(const struct ipt_entry *fw,
 
 		if (fw->ip.iniface[0] != '\0') {
 			strcat(iface, fw->ip.iniface);
-			/* If it doesn't compare the nul-term, it's a
-			   wildcard. */
-			if (fw->ip.iniface_mask[strlen(fw->ip.iniface)] == 0)
-				strcat(iface, "+");
 		}
 		else if (format & FMT_NUMERIC) strcat(iface, "*");
 		else strcat(iface, "any");
@@ -1182,10 +1202,6 @@ print_firewall(const struct ipt_entry *fw,
 
 		if (fw->ip.outiface[0] != '\0') {
 			strcat(iface, fw->ip.outiface);
-			/* If it doesn't compare the nul-term, it's a
-			   wildcard. */
-			if (fw->ip.outiface_mask[strlen(fw->ip.outiface)] == 0)
-				strcat(iface, "+");
 		}
 		else if (format & FMT_NUMERIC) strcat(iface, "*");
 		else strcat(iface, "any");
@@ -1560,7 +1576,7 @@ static char *get_modprobe(void)
 	return NULL;
 }
 
-static int iptables_insmod(const char *modname, const char *modprobe)
+int iptables_insmod(const char *modname, const char *modprobe)
 {
 	char *buf = NULL;
 	char *argv[3];
